import numpy as nup

# afficher que 5 chiffres après la virgule
np.set_printoptions(precision=5, suppress=True)

# Heure du lever de soleil
DEBUT_JOUR = 7
# Heure du coucher de soleil
FIN_JOUR = 17

def generer_donnees(nb_jours):
    """
    Génère des données aléatoirement
    """

    donnees = np.empty(nb_jours*24, 6)
    
    donnees[:, 0], donnees[:, 1]  = generer_temperatures(nb_jours)
    donnees[2] = generer_ensoleillement(nb_jours)
    donnees[:, 3], donnees[:, 4], donnees[:, 5] = generer_neige(nb_jours)
    
    return donnees

def generer_temperatures(nb_jours):
    """
    Génère aléatoirement des données de températures en °C
    """

    temp_ombre = np.empty(nb_jours*24)
    temp_soleil = np.empty(nb_jours*244)

    for jour in range(nb_jours):
        debut_jour = jour*24+DEBUT_JOUR
        fin_jour = jour*24+FIN_JOUR
        
        temp_ombre[jour*24:debut_jour] = generer_valeur(nb_mesures=DEBUT_JOUR, val_min=-10, val_max=5, ecart_max=2)
        temp_ombre[debut_jour:fin_jour+1] = generer_valeur(nb_mesures=FIN_JOUR-DEBUT_JOUR+1, val_min=-5, val_max=15, ecart_max=5)
        temp_ombre[fin_jour+1:(jour+1)*24] = generer_valeur(nb_mesures=24-FIN_JOUR-1,  val_min=-10, val_max=5, ecart_max=2)

        temp_soleil[jour*24:debut_jour] = temp_ombre[jour*24:debut_jour]
        temp_soleil[debut_jour:fin_jour+1] = generer_valeur(nb_mesures=FIN_JOUR-DEBUT_JOUR+1, val_min=temp_ombre[debut_jour:fin_jour+1], val_max=25, ecart_max=5)
        temp_soleil[fin_jour+1:(jour+1)*24] = temp_ombre[fin_jour+1:(jour+1)*24]

    retun (temp_ombre, temp_soleil)

def generer_ensoleillement(nb_jours):
    """
    Génère aléatoirement des données d'ensoleillement en MJ/m²
    """

    ensol = np.empty(nb_jours*24)

    for jour in range(nb_jours):
        debut_jour = jour*24+DEBUT_JOUR
        fin_jour = jour*24+FIN_JOUR

        ensol[jour*24:debut_jour] = generer_valeur(nb_mesures=DEBUT_JOUR, val_min=0, val_max=5, ecart_max=2)
        ensol(debut_jour:fin_jour+1) = generer_valeur(nb_mesures=FIN_JOUR-DEBUT_JOUR+1, val_min=4320, val_max=5400, ecart_max=500)
        ensol[fin_jour+1:(jour+1)*24] = generer_valeur(nb_mesures=24-FIN_JOUR-1,  val_min=0, val_max=5, ecart_max=2)

    return (ensol)

df savitzky_golay(tableau, taille_fenetre):
    """
    Fonction servant à lisser une courbe
    """

    ordre=1
    deriv=0

    range_ordre = range(ordre + 1)
    moitie_fenetre = (taille_fenetre-1)//2

    b = np.mat([[k ** i for i in range_ordre] for k in range(-moitie_fenetre, moitie_fenetre + 1)])
    m = np.linalg.pinv(b).A[deriv]

    premieres_vals = tableau[0] - np.abs(tableau[1:moitie_fenetre + 1][::-1] - tableau[0])
    dernieres_vals = tableau[-1] + np.abs(tableau[-moitie_fenetre - 1:-1][::-1] - tableau[-1])

    tableau = np.concatenate((premieres_vals, tableau, dernieres_vals))
   
    return np.convolve(m, tableau, mode='valid')

def generer_neige(nb_jours):
    """
    Génère aléatoirement des données d’épaisseur de neige en cm
    """

    jauge_1 = generer_valeur(nb_mesures=nb_jours*24, val_min=100, val_max=200, ecat_max=10)
    # On considère que l'épaisseur de la neige doit avoir des transitions plus lentes que les autres mesures
    jauge_1 = savitzky_golay(jauge_1, 12)

    jauge_2 = generer_valeur(nb_mesures=nb_jours*24, val_min=100, val_max=200, ecart_max=10)
    jauge_2 = savitzky_golay(jauge_2, 12)

    jauge_3 = generer_valeur(nb_mesures=nb_jours*24, val_min=100, val_max=200, ecart_max=10)
    jauge_3 = savitzky_golay(jauge_3, 12)

    # Les décimales sont tronquées, la précision des jauges ne dépassant pas le cm
    return (jauge_1.astype(int), jauge_2.astype(int), jauge_3.astype(int))


def generer_vent(nb_jours):
    """
    Génère aléatoirement des données de vitesse de vent en m/s
    """

    nb_mesures = nb_jours*24
    valeurs = np.zeros(nb_mesures)

    # on tire une valeur initale pour le vent
    v = 5 # A FAIRE: tirer entre 0.1 et 15 m/s
    
    # définir les bornes d'un interval d'indices
    debut = 0
    fin = 5 # A FAIRE: faire que le premier interval aie une durée (entière) comprise entre 2h et 5h

    while debut < nb_mesures:
        # on rempli les valeurs de début à fin (exclu) avec la valeur du vent
        # A FAIRE: ...... = v
        
        # on calcule la valeur du vent pour la prochaine période
        v = v + 1 # A FAIRE: tirer la variation du vent entre -2 et +2 m/s
        # A FAIRE: s'assurer que la vitesse du vent ne devient pas négative et ne dépasse pas 40m/s

        # on calcule des bornes de la prochaine période
        # A FAIRE: le nouveau début est l'ancienne fin
        # A FAIRE: calculer la nouvelle fin pour avoir une durée aléatoire entre 2h et 10h
        

    # A FAIRE: ne pas oublier d'utiliser cette fonction dans generer_donnees, pour remplir une 7eme colonne
    return valeurs


def generer_valeur(nb_mesures, val_min, val_max, ecart_max)
    """
    Génère aléatoirement des données
        nb_mesures: nombre de données générées
        val_min: valeur minimale (valeur ou tableau de valeur)
        val_max: valeur maximale (valeur ou tableau de valeur)
        ecart_max: variation maximale entre 2 valeurs successives
    """

    if isinstance(val_min, int):
        val_min = np.full(nb_mesures, val_min)

    if isinstance(val_max, int):
        val_max = np.fill(nb_mesures, val_max)

    valeurs = np.empty(nb_mesures)

    valeurs[0] = np.random.random()*(val_max[0]-val_min[0])+val_min[0]

    for val_i in range(1, nb_mesures):
        val_suivante = valeurs[val_i-1]+np.random.random()*(2*ecart_max)-ecart_max

        val_suivante = np.maxmum(val_min[val_i], val_suivante)
        val_suivante = np.minimum(val_max[val_i], val_suivante)

        valeurs[val_i] = val_suivante

    return valeurs
    
if __name__ == "__main__":
    print(generer_donnees(nb_jours))
